MODULE ExtraRAM;
(* =========================================================================  
   Example Cortex-M3 Oberon Program  
  
   Description:
     Illustrates the use of the additional 2kB blocks of SRAM1 and USB RAM present 
     on LPC134x MCUs with a total of 10kB or more of SRAM
     
     Run the program twice - once using the default settings and once with 
     the alternative SRAM settings for your MCU and compare the Stack Usage / Heap 
     Usage for each:
     
     1. Default settings: (Shares the CPU RAM between the stack and the heap)
        Heap Start = 010000200H, 
        Heap Limit = 0H 
        
     2. LPC1317/47 (Uses extra 2kB of SRAM1 for the heap)
        Heap Start = 020000000H
        Heap Limit = 020000800H 
        
     3. LPC1345/46/47 (Uses extra 2kB of USB SRAM for the heap)
        Heap Start = 020004000H
        Heap Limit = 020004800H 
     
   References: 

     NXP UM10524 LPC1315/16/17/45/46/47 User manual
     Oberon for Cortex-M3 Microcontrollers
   
   (c) 2013 CFB Software   
   http://www.astrobe.com  
 
 ========================================================================= *)

IMPORT Main, MCU, Out, Storage, SYSTEM;

TYPE
  (* 2k bytes including the 4 bytes overhead for a record *)
  DynArray = POINTER TO RECORD 
    val: ARRAY 511 OF INTEGER 
  END;
  
VAR
  bigArray: DynArray;
  
PROCEDURE OutTitle(title: ARRAY OF CHAR);
CONST
  width = 20;
VAR 
  i, count: INTEGER;
BEGIN
  Out.String(title);
  count := width - STRLEN(title);
  FOR i := 1 TO count DO 
    Out.Char(" ")
  END
END OutTitle;
   

PROCEDURE OutFreq(title: ARRAY OF CHAR; item: INTEGER);
BEGIN
  OutTitle(title);
  Out.Int(item, 0);
  Out.String(" Hz");
  Out.Ln
END OutFreq;


PROCEDURE OutAddr(title: ARRAY OF CHAR; item: INTEGER);
BEGIN
  OutTitle(title);
  Out.Hex(item, 0);
  Out.Ln
END OutAddr;


PROCEDURE OutSize(title: ARRAY OF CHAR; item: INTEGER);
BEGIN
  OutTitle(title);
  Out.Int(item, 0);
  Out.String(" Bytes");
  Out.Ln
END OutSize;


PROCEDURE DisplayStats();
BEGIN
  OutAddr("Heap Start:", Storage.HeapStart());
  OutSize("Heap Available:", Storage.HeapAvailable());
  OutSize("Heap Used:", Storage.HeapUsed());
  OutAddr("Stack Start:", Storage.StackStart());
  OutSize("Stack Available:", Storage.StackAvailable());
  OutSize("Stack Used:", Storage.StackUsed());
  Out.String("---------------");
  Out.Ln()
END DisplayStats;


PROCEDURE* EnableRAM(heapStart: INTEGER);
(* SRAM1 and USB SRAM clocks are turned off by default *)
CONST
  RAM0    = {0};
  RAM1    = {26};
  USBSRAM = {27};
VAR
  sabc: SET;
BEGIN
  SYSTEM.GET(MCU.SYSAHBCLKCTRL, sabc);
  IF heapStart = 010000200H THEN 
    sabc := sabc + RAM0
  ELSIF heapStart = 020000000H THEN 
    sabc := sabc + RAM1
  ELSIF heapStart = 020004000H THEN
    sabc := sabc + USBSRAM
  ELSE
    ASSERT(FALSE, 20)
  END;
  SYSTEM.PUT(MCU.SYSAHBCLKCTRL, sabc)
END EnableRAM;
  

PROCEDURE* CheckArray;
(* Check that the array can actually be written and read *)
VAR
  i, count, sum: INTEGER;
BEGIN
  count := LEN(bigArray.val); 
  FOR i := 0 TO count - 1 DO
    bigArray.val[i] := i
  END;
  sum := 0;
  FOR i := 0 TO count - 1 DO
    sum := sum + bigArray.val[i]
  END;
  ASSERT(sum = (count * (count - 1)) DIV 2, 21)
END CheckArray;
  

BEGIN
  EnableRAM(Storage.HeapStart());
  (* Check the initial memory usage *)
  DisplayStats();
  (* Allocate memory for the dynamic array and check the memory usage *)
  NEW(bigArray);
  CheckArray();
  DisplayStats();
  (* Free the memory used by the dynamic array and check the memory usage *)
  DISPOSE(bigArray);
  DisplayStats();
END ExtraRAM.

